@@ -75,11 +75,14 @@ PythonCodeCompletionContext::CompletionContextType PythonCodeCompletionContext::
 std::unique_ptr<ExpressionVisitor> visitorForString(QString str, DUContext* context,
                                                     CursorInRevision scanUntil = CursorInRevision::invalid())
 {
-    ENSURE_CHAIN_NOT_LOCKED
+    ENSURE_CHAIN_READ_LOCKED
+    if ( !context ) {
+        return nullptr;
+    }
     AstBuilder builder;
     CodeAst::Ptr tmpAst = builder.parse({}, str);
     if ( ! tmpAst ) {
-        return std::unique_ptr<ExpressionVisitor>(nullptr);
+        return nullptr;
     }
     ExpressionVisitor* v = new ExpressionVisitor(context);
     v->enableGlobalSearching();
@@ -140,15 +143,15 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::functionCallI
     ItemList resultingItems;
 
     // gather additional items to show above the real ones (for parameters, and stuff)
-    FunctionDeclaration* functionCalled = 0;
-    auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
+    FunctionDeclaration* functionCalled = nullptr;
     DUChainReadLocker lock;
+    auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
     if ( ! v || ! v->lastDeclaration() ) {
         qCWarning(KDEV_PYTHON_CODECOMPLETION) << "Did not receive a function declaration from expression visitor! Not offering call tips.";
         qCWarning(KDEV_PYTHON_CODECOMPLETION) << "Tried: " << m_guessTypeOfExpression;
         return resultingItems;
     }
-    functionCalled = Helper::functionDeclarationForCalledDeclaration(v->lastDeclaration()).first.data();
+    functionCalled = Helper::functionForCalled(v->lastDeclaration().data()).declaration;
 
     auto current = Helper::resolveAliasDeclaration(functionCalled);
     QList<Declaration*> calltips;
@@ -203,7 +206,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::defineItems()
     if ( ! klass ) {
         return resultingItems;
     }
-    QList<DUContext*> baseClassContexts = Helper::internalContextsForClass(
+    auto baseClassContexts = Helper::internalContextsForClass(
         klass->type<StructureType>(), m_duContext->topContext()
     );
     // This class' context is put first in the list, so all functions existing here
@@ -253,6 +256,9 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::raiseItems()
     DUChainReadLocker lock;
     ItemList resultingItems;
     ReferencedTopDUContext ctx = Helper::getDocumentationFileContext();
+    if ( !ctx ) {
+        return {};
+    }
     QList< Declaration* > declarations = ctx->findDeclarations(QualifiedIdentifier("BaseException"));
     if ( declarations.isEmpty() || ! declarations.first()->abstractType() ) {
         qCDebug(KDEV_PYTHON_CODECOMPLETION) << "No valid exception classes found, aborting";
@@ -261,7 +267,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::raiseItems()
     Declaration* base = declarations.first();
     IndexedType baseType = base->abstractType()->indexed();
     QList<DeclarationDepthPair> validDeclarations;
-    ClassDeclaration* current = 0;
+    ClassDeclaration* current = nullptr;
     StructureType::Ptr type;
     auto decls = m_duContext->topContext()->allDeclarations(CursorInRevision::invalid(), m_duContext->topContext());
     foreach ( const DeclarationDepthPair d, decls ) {
@@ -302,9 +308,7 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::inheritanceIt
     QList<DeclarationDepthPair> declarations;
     if ( ! m_guessTypeOfExpression.isEmpty() ) {
         // The class completion is a member access
-        lock.unlock();
         auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
-        lock.lock();
         if ( v ) {
             TypePtr<StructureType> cls = StructureType::Ptr::dynamicCast(v->lastType());
             if ( cls && cls->declaration(m_duContext->topContext()) ) {
@@ -334,8 +338,8 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::inheritanceIt
 PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::memberAccessItems()
 {
     ItemList resultingItems;
-    auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
     DUChainReadLocker lock;
+    auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data());
     if ( v ) {
         if ( v->lastType() ) {
             qCDebug(KDEV_PYTHON_CODECOMPLETION) << v->lastType()->toString();
@@ -522,8 +526,8 @@ PythonCodeCompletionContext::ItemList PythonCodeCompletionContext::generatorItem
     ItemList resultingItems;
     QList<KeywordItem*> items;
 
-    auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data(), m_position);
     DUChainReadLocker lock;
+    auto v = visitorForString(m_guessTypeOfExpression, m_duContext.data(), m_position);
     if ( ! v || v->unknownNames().isEmpty() ) {
         return resultingItems;
     }
@@ -641,8 +645,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::getMissingIncludeI
         return items;
     }
 
-    Declaration* existing = Helper::declarationForName(QualifiedIdentifier(components.first()),
-                                                       RangeInRevision(m_position, m_position),
+    Declaration* existing = Helper::declarationForName(components.first(), m_position,
                                                        DUChainPointer<const DUContext>(m_duContext.data()));
     if ( existing ) {
         // There's already a declaration for the first component; no need to suggest it
@@ -678,7 +681,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToI
     QList<CompletionTreeItemPointer> items;
     
     DeclarationPointer currentDeclaration;
-    Declaration* checkDeclaration = 0;
+    Declaration* checkDeclaration = nullptr;
     int count = declarations.length();
     for ( int i = 0; i < count; i++ ) {
         if ( maxDepth && maxDepth > declarations.at(i).second ) {
@@ -687,7 +690,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::declarationListToI
         }
         currentDeclaration = DeclarationPointer(declarations.at(i).first);
         
-        PythonDeclarationCompletionItem* item = 0;
+        PythonDeclarationCompletionItem* item = nullptr;
         checkDeclaration = Helper::resolveAliasDeclaration(currentDeclaration.data());
         if ( ! checkDeclaration ) {
             continue;
@@ -776,7 +779,7 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::getCompletionItems
         return QList<CompletionTreeItemPointer>();
     }
     // the PublicOnly will filter out non-explictly defined __get__ etc. functions inherited from object
-    QList<DUContext*> searchContexts = Helper::internalContextsForClass(cls, m_duContext->topContext(), Helper::PublicOnly);
+    auto searchContexts = Helper::internalContextsForClass(cls, m_duContext->topContext(), Helper::PublicOnly);
     QList<DeclarationDepthPair> keepDeclarations;
     foreach ( const DUContext* currentlySearchedContext, searchContexts ) {
         qCDebug(KDEV_PYTHON_CODECOMPLETION) << "searching context " << currentlySearchedContext->scopeIdentifier() << "for autocompletion items";
@@ -894,10 +897,10 @@ QList<CompletionTreeItemPointer> PythonCodeCompletionContext::findIncludeItems(Q
 
 DUContext* PythonCodeCompletionContext::internalContextForDeclaration(TopDUContext* topContext, QStringList remainingIdentifiers)
 {
-    Declaration* d = 0;
+    Declaration* d = nullptr;
     DUContext* c = topContext;
     if ( ! topContext ) {
-        return 0;
+        return nullptr;
     }
     if ( remainingIdentifiers.isEmpty() ) {
         return topContext;
@@ -906,7 +909,7 @@ DUContext* PythonCodeCompletionContext::internalContextForDeclaration(TopDUConte
         QList< Declaration* > decls = c->findDeclarations(QualifiedIdentifier(remainingIdentifiers.first()));
         remainingIdentifiers.removeFirst();
         if ( decls.isEmpty() ) {
-            return 0;
+            return nullptr;
         }
         d = decls.first();
         if ( (c = d->internalContext()) ) {
@@ -914,15 +917,15 @@ DUContext* PythonCodeCompletionContext::internalContextForDeclaration(TopDUConte
                 return c;
             }
         }
-        else return 0;
+        else return nullptr;
         
     } while ( d && ! remainingIdentifiers.isEmpty() );
-    return 0;
+    return nullptr;
 }
 
 QList<CompletionTreeItemPointer> PythonCodeCompletionContext::includeItemsForSubmodule(QString submodule)
 {
-    QList<QUrl> searchPaths = Helper::getSearchPaths(m_workingOnDocument);
+    auto searchPaths = Helper::getSearchPaths(m_workingOnDocument);
     
     QStringList subdirs;
     if ( ! submodule.isEmpty() ) {
@@ -1026,7 +1029,7 @@ PythonCodeCompletionContext::PythonCodeCompletionContext(DUContextPointer contex
     : CodeCompletionContext(context, text, position, depth)
     , m_operation(PythonCodeCompletionContext::DefaultCompletion)
     , m_itemTypeHint(NoHint)
-    , m_child(0)
+    , m_child(nullptr)
     , m_followingText(followingText)
     , m_position(position)
 {
