@@ -319,10 +319,6 @@ void createArgumentList(Declaration* dec_, QString& ret, QList< QVariant >* high
         if (DUChainUtils::getArgumentContext(dec))
             parameters = DUChainUtils::getArgumentContext(dec)->localDeclarations();
 
-        uint defaultParamNum = 0;
-
-        int firstDefaultParam = parameters.count() - decl->defaultParametersSize();
-
         ret = '(';
         bool first = true;
         int num = 0;
@@ -332,7 +328,10 @@ void createArgumentList(Declaration* dec_, QString& ret, QList< QVariant >* high
             // the function is a class method, and its first argument is "self". Don't display that.
             skipFirst = true;
         }
-        
+
+        uint defaultParamNum = 0;
+        int firstDefaultParam = parameters.count() - decl->defaultParametersSize() - skipFirst;
+
         // disable highlighting when in default arguments, it doesn't make much sense then
         bool disableHighlighting = false;
         
@@ -442,7 +441,7 @@ StringFormatter::StringFormatter(const QString &string)
 
 bool StringFormatter::isInsideReplacementVariable(int cursorPosition) const
 {
-    return getReplacementVariable(cursorPosition) != NULL;
+    return getReplacementVariable(cursorPosition) != nullptr;
 }
 
 const ReplacementVariable *StringFormatter::getReplacementVariable(int cursorPosition) const
@@ -455,7 +454,7 @@ const ReplacementVariable *StringFormatter::getReplacementVariable(int cursorPos
         index++;
     }
 
-    return 0;
+    return nullptr;
 }
 
 RangeInString StringFormatter::getVariablePosition(int cursorPosition) const
